package ru.dbotthepony.mc.otm.worldgen.feature

import com.mojang.serialization.Codec
import com.mojang.serialization.MapCodec
import net.minecraft.core.BlockPos
import net.minecraft.core.Holder
import net.minecraft.resources.RegistryFileCodec
import net.minecraft.world.level.levelgen.feature.ConfiguredFeature
import net.neoforged.bus.api.IEventBus
import net.neoforged.neoforge.registries.DataPackRegistryEvent
import ru.dbotthepony.mc.otm.registry.MBuiltInRegistries
import ru.dbotthepony.mc.otm.registry.MDeferredRegister
import ru.dbotthepony.mc.otm.registry.MRegistries
import ru.dbotthepony.mc.otm.worldgen.EnhancedPlacementContext

abstract class EnhancedFeature<FC>(codec: Codec<FC>) {
	abstract fun place(context: EnhancedPlacementContext, config: FC, positions: List<BlockPos>, allPositions: List<BlockPos>): Boolean

	data class Configured<F : EnhancedFeature<FC>, FC>(val feature: F, val config: FC) {
		fun place(context: EnhancedPlacementContext, positions: List<BlockPos>, allPositions: List<BlockPos>): Boolean {
			return feature.place(context, config, positions, allPositions)
		}
	}

	val codec: MapCodec<Configured<*, FC>> = codec.fieldOf("config").xmap({ Configured(this, it) }, { it.config })
	fun configure(config: FC) = Configured(this, config)

	object Wrapper : EnhancedFeature<Holder<ConfiguredFeature<*, *>>>(ConfiguredFeature.CODEC) {
		override fun place(
			context: EnhancedPlacementContext,
			config: Holder<ConfiguredFeature<*, *>>,
			positions: List<BlockPos>,
			allPositions: List<BlockPos>
		): Boolean {
			var any = false
			positions.forEach { any = config.value().place(context.level, context.generator, context.random, it) || any }
			return any
		}

		fun configure(config: ConfiguredFeature<*, *>) = configure(Holder.direct(config))
	}

	companion object {
		private val registrar = MDeferredRegister(MRegistries.FEATURE)

		val DIRECT_CODEC: Codec<Configured<*, *>> by lazy {
			MBuiltInRegistries.FEATURE.byNameCodec().dispatch({ it.feature }, { it.codec })
		}

		val CODEC: Codec<Holder<Configured<*, *>>> by lazy {
			RegistryFileCodec.create(MRegistries.CONFIGURED_FEATURE, DIRECT_CODEC)
		}

		init {
			registrar.register("wrapper") { Wrapper }
		}

		private fun registerRegistry(event: DataPackRegistryEvent.NewRegistry) {
			event.dataPackRegistry(MRegistries.CONFIGURED_FEATURE, DIRECT_CODEC)
		}

		internal fun register(bus: IEventBus) {
			registrar.register(bus)
			bus.addListener(::registerRegistry)
		}
	}
}
